/* iod-telemetry Database.sql version 2019.05.12.1 stkolev */
with
    v$metricname as (
        /* version 3 */
        select
            ''              as project
            ,'oracle_db'    as fleet
            ,host_name      as host
            ,'"lifecycle":"'||
                case
                    when
                        (regexp_like(instance_name,'(9[0-9])') and host_name not like '%.r1') or
                        regexp_like(host_name,'(dev|sandbox)') or
                        regexp_like(lower(sys_context('USERENV','DB_DOMAIN')),'(dev|sand)')
                        then 'DEV'
                    when
                        (regexp_like(instance_name,'(9[0-9])') and host_name like '%.r1%') or
                        regexp_like(host_name,'(test|tst)') or
                        regexp_like(lower(sys_context('USERENV','DB_DOMAIN')),'(test|tst)')
                        then 'TEST'
                    else 'PROD'
                end || '"' ||
            ',"hostclass":"'|| nvl(regexp_substr(host_name,'^([^.]*)-',1,1,NULL,1),'N/A') || '"' ||
            ',"instance_name":"'     || instance_name    || '"' ||
            ',"db_name":"'           || nvl(name,'N/A')             || '"' ||
            ',"db_domain":"'         || nvl(sys_context('USERENV','DB_DOMAIN'),'N/A') || '"' ||
            ',"db_unique_name":"'    || db_unique_name   || '"' ||
            ',"database_role":"'     || database_role    || '"' ||
            ',"fsfo_role":"'||
                case
                    when database_role = 'PRIMARY' then 'PRIMARY'
                    when database_role = 'PHYSICAL STANDBY' and lower(db_unique_name)  = lower(fs_failover_current_target) then 'STANDBY'
                    when database_role = 'PHYSICAL STANDBY' and lower(db_unique_name) != lower(fs_failover_current_target) then 'BYSTANDER'
                    else 'NONE'
                end || '"' as metric_dimensions
        from v$database, v$instance
    ),
    v$metric_history as (
        select * from v$database
    ),
    v$metric as (
        select
            'Database' as metric_name
            ,',"log_mode": "'                   || nvl(to_char(log_mode                     ),'null') || '"' ||
            ',"open_mode": "'                   || nvl(to_char(open_mode                    ),'null') || '"' ||
            ',"protection_mode": "'             || nvl(to_char(protection_mode              ),'null') || '"' ||
            ',"dataguard_broker": "'            || nvl(to_char(dataguard_broker             ),'null') || '"' ||
            ',"force_logging": "'               || nvl(to_char(force_logging                ),'null') || '"' ||
            ',"platform_name": "'               || nvl(to_char(platform_name                ),'null') || '"' ||
            ',"flashback_on": "'                || nvl(to_char(flashback_on                 ),'null') || '"' ||
            ',"fs_failover_status": "'          || nvl(to_char(fs_failover_status           ),'null') || '"' ||
            --',"fs_failover_current_target": "'  || nvl(to_char(fs_failover_current_target   ),'null') || '"' ||
            --',"fs_failover_threshold": "'       || nvl(to_char(fs_failover_threshold        ),'null') || '"' ||
            --',"fs_failover_observer_present":"' || nvl(to_char(fs_failover_observer_present ),'null') || '"' ||
            --',"fs_failover_observer_host": "'   || nvl(to_char(fs_failover_observer_host    ),'null') || '"' ||
            ',"primary_db_unique_name": "'      || nvl(to_char(primary_db_unique_name       ),'null') || '"' as metric_dimensions
            ,1 as metric_value
        from v$metric_history
    )
select
    v$metricname.project        as project
    ,v$metricname.fleet         as fleet
    ,v$metricname.host          as host
    ,v$metric.metric_name       as metric_name
    ,v$metric.metric_value      as metric_value
    ,'{'||v$metricname.metric_dimensions||v$metric.metric_dimensions||'}' as metric_dimensions
from v$metricname,v$metric
