/* iod-telemetry RecoveryFileDest.sql version 2019.05.12.1 stkolev */
with
    v$metricname as (
        /* version 3 */
        select
            ''              as project
            ,'oracle_db'    as fleet
            ,host_name      as host
            ,'"lifecycle":"'||
                case
                    when
                        (regexp_like(instance_name,'(9[0-9])') and host_name not like '%.r1') or
                        regexp_like(host_name,'(dev|sandbox)') or
                        regexp_like(lower(sys_context('USERENV','DB_DOMAIN')),'(dev|sand)')
                        then 'DEV'
                    when
                        (regexp_like(instance_name,'(9[0-9])') and host_name like '%.r1%') or
                        regexp_like(host_name,'(test|tst)') or
                        regexp_like(lower(sys_context('USERENV','DB_DOMAIN')),'(test|tst)')
                        then 'TEST'
                    else 'PROD'
                end || '"' ||
             ',"hostclass":"'|| nvl(regexp_substr(host_name,'^([^.]*)-',1,1,NULL,1),'N/A') || '"' ||
            ',"instance_name":"'     || instance_name    || '"' ||
            ',"db_name":"'           || nvl(name,'N/A')             || '"' ||
            ',"db_domain":"'         || nvl(sys_context('USERENV','DB_DOMAIN'),'N/A') || '"' ||
            ',"db_unique_name":"'    || db_unique_name   || '"' ||
            ',"database_role":"'     || database_role    || '"' ||
            ',"fsfo_role":"'||
                case
                    when database_role = 'PRIMARY' then 'PRIMARY'
                    when database_role = 'PHYSICAL STANDBY' and lower(db_unique_name)  = lower(fs_failover_current_target) then 'STANDBY'
                    when database_role = 'PHYSICAL STANDBY' and lower(db_unique_name) != lower(fs_failover_current_target) then 'BYSTANDER'
                    else 'NONE'
                end || '"' as metric_dimensions
        from v$database, v$instance
    ),
    v$metric_history as (
            select * from v$recovery_file_dest
    ),
    v$metric as (
        select
            'RecoveryFileDest.' || metric_name as metric_name
            ,',"name":"'    || nvl(to_char(name),'null') || '"' as metric_dimensions
            ,metric_value as metric_value
        from (
            select
                name
                ,space_limit                                                                as "SpaceLimit"
                ,space_used                                                                 as "SpaceUsed"
                ,space_used - space_reclaimable                                             as "EffectiveSpaceUsed"
                ,space_reclaimable                                                          as "SpaceReclaimable"
                ,space_limit - space_used                                                   as "SpaceFree"
                ,space_limit - (space_used - space_reclaimable)                             as "EffectiveSpaceFree"
                ,number_of_files                                                            as "NumberOfFiles"
                ,100-round(((space_used)/space_limit)*100,2)                                as "PercentFree"
                ,100-round(((space_used-space_reclaimable)/space_limit)*100,2)              as "EffectivePercentFree"
                ,round(((space_used)/space_limit)*100,2)                                    as "PercentUsed"
                ,round(((space_used-space_reclaimable)/space_limit)*100,2)                  as "EffectivePercentUsed"
            from
                v$recovery_file_dest
        ) unpivot (
            metric_value FOR metric_name IN (
                 "SpaceLimit"
                ,"SpaceUsed"
                ,"EffectiveSpaceUsed"
                ,"SpaceReclaimable"
                ,"SpaceFree"
                ,"EffectiveSpaceFree"
                ,"NumberOfFiles"
                ,"PercentFree"
                ,"EffectivePercentFree"
                ,"PercentUsed"
                ,"EffectivePercentUsed"
            )
        )
    )
select
    v$metricname.project                        as project
    ,v$metricname.fleet                         as fleet
    ,v$metricname.host                          as host
    ,v$metric.metric_name                       as metric_name
    ,v$metric.metric_value                      as metric_value
    ,'{'||v$metricname.metric_dimensions||v$metric.metric_dimensions||'}' as metric_dimensions
from v$metricname,v$metric
